import { FormBuilder, FormGroup, AbstractControl, AbstractControlOptions, ValidatorFn, AsyncValidatorFn, FormControl, FormArray } from '@angular/forms';
declare type UnpackArrayType<T> = T extends (infer R)[] ? R : T;
declare type TypedControllerValue<T> = (T extends TypedFormGroup<unknown> ? T['value'] : T extends TypedFormArray<unknown> ? T['value'] : T);
declare type TypedController<T> = (T extends FormArray ? TypedFormArray<UnpackArrayType<T['value']>> : (T extends FormGroup ? TypedFormGroup<T['value']> : TypedAbstractControl<T>));
export declare abstract class TypedAbstractControl<T> extends AbstractControl {
    readonly value: T;
    abstract setValue(value: T, options?: Object): void;
    abstract patchValue(value: T, options?: Object): void;
    abstract reset(value?: T, options?: Object): void;
}
export declare abstract class TypedFormControl<T> extends FormControl {
    readonly value: T;
    reset(value: Parameters<TypedAbstractControl<T>['reset']>[0], options?: Parameters<FormControl['reset']>[1]): void;
    patchValue(value: Parameters<TypedAbstractControl<T>['patchValue']>[0], options?: Parameters<FormControl['patchValue']>[1]): void;
    setValue(value: Parameters<TypedAbstractControl<T>['setValue']>[0], options?: Parameters<FormControl['setValue']>[1]): void;
}
export declare abstract class TypedFormGroup<T> extends FormGroup {
    readonly value: {
        [P in keyof T]: TypedControllerValue<T[P]>;
    };
    controls: {
        [P in keyof T]: TypedController<T[P]>;
    };
    patchValue(value: {
        [P in keyof Partial<T>]: TypedAbstractControl<T[P]>;
    }, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    setValue(value: {
        [P in keyof Partial<T>]: TypedAbstractControl<T[P]>;
    }, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
}
export declare abstract class TypedFormArray<T> extends FormArray {
    readonly value: TypedControllerValue<T>[];
    controls: TypedController<T>[];
    reset(value: Parameters<TypedAbstractControl<T>['reset']>[0], options?: Parameters<FormControl['reset']>[1]): void;
    push(control: TypedAbstractControl<T>, options?: {
        emitEvent?: boolean;
    }): void;
    insert(index: number, control: TypedAbstractControl<T>, options?: {
        emitEvent?: boolean;
    }): void;
    setControl(index: number, control: TypedAbstractControl<T>, options?: {
        emitEvent?: boolean;
    }): void;
    setValue(value: Parameters<TypedAbstractControl<T>['setValue']>[0][], options?: Parameters<FormControl['setValue']>[1]): void;
    patchValue(value: Parameters<TypedAbstractControl<T>['patchValue']>[0][], options?: Parameters<FormControl['patchValue']>[1]): void;
    getRawValue(): T[];
}
export declare class NgxTypedFormBuilderService {
    fb: FormBuilder;
    constructor(fb: FormBuilder);
    control<T>(formState: any, validatorOrOpts?: ValidatorFn | ValidatorFn[] | AbstractControlOptions | null, asyncValidator?: AsyncValidatorFn | AsyncValidatorFn[] | null): TypedFormControl<T>;
    group<T>(controlsConfig: {
        [key in keyof T]: any;
    }, options?: AbstractControlOptions | {
        [key: string]: any;
    } | null): TypedFormGroup<T>;
    array(controlsConfig: any[], validatorOrOpts?: ValidatorFn | ValidatorFn[] | AbstractControlOptions | null, asyncValidator?: AsyncValidatorFn | AsyncValidatorFn[] | null): FormArray;
}
export {};
